@@ -22,7 +22,6 @@
 #include <string.h>
 
 #include "bson-context-private.h"
-#include "bson-md5.h"
 #include "bson-oid.h"
 #include "bson-string.h"
 
@@ -134,7 +133,7 @@ bson_oid_init_sequence (bson_oid_t *oid,         /* OUT */
  *
  *       Generates bytes for a new bson_oid_t and stores them in @oid. The
  *       bytes will be generated according to the specification and includes
- *       the current time, first 3 bytes of MD5(hostname), pid (or tid), and
+ *       the current time, a 3 byte hash of the hostname, pid (or tid), and
  *       monotonic counter.
  *
  *       The bson_oid_t generated by this function is not guaranteed to be
