@@ -11,7 +11,10 @@ SEXP R_mongo_get_default_database(SEXP ptr_client) {
   mongoc_client_t *client = r2client(ptr_client);
   mongoc_database_t *db = mongoc_client_get_default_database(client);
   if(db){
-    return Rf_mkString(mongoc_database_get_name(db));
+    SEXP out = PROTECT(Rf_mkString(mongoc_database_get_name(db)));
+    mongoc_database_destroy(db);
+    UNPROTECT(1);
+    return out;
   } else {
     return R_NilValue;
   }
@@ -173,8 +176,7 @@ SEXP R_mongo_collection_find(SEXP ptr_col, SEXP ptr_query, SEXP ptr_opts) {
   mongoc_collection_t *col = r2col(ptr_col);
   bson_t *query = r2bson(ptr_query);
   bson_t *opts = r2bson(ptr_opts);
-  mongoc_read_prefs_t * readpref = mongoc_read_prefs_new(MONGOC_READ_PRIMARY);
-  mongoc_cursor_t *c = mongoc_collection_find_with_opts(col, query, opts, readpref);
+  mongoc_cursor_t *c = mongoc_collection_find_with_opts(col, query, opts, NULL);
   return cursor2r(c, ptr_col);
 }
 
