@@ -49,10 +49,11 @@ mongoc_find_and_modify_opts_set_sort (mongoc_find_and_modify_opts_t *opts,
    BSON_ASSERT (opts);
 
    if (sort) {
-      _mongoc_bson_destroy_if_set (opts->sort);
+      bson_destroy (opts->sort);
       opts->sort = bson_copy (sort);
       return true;
    }
+
    return false;
 }
 
@@ -77,10 +78,11 @@ mongoc_find_and_modify_opts_set_update (mongoc_find_and_modify_opts_t *opts,
    BSON_ASSERT (opts);
 
    if (update) {
-      _mongoc_bson_destroy_if_set (opts->update);
+      bson_destroy (opts->update);
       opts->update = bson_copy (update);
       return true;
    }
+
    return false;
 }
 
@@ -105,10 +107,11 @@ mongoc_find_and_modify_opts_set_fields (mongoc_find_and_modify_opts_t *opts,
    BSON_ASSERT (opts);
 
    if (fields) {
-      _mongoc_bson_destroy_if_set (opts->fields);
+      bson_destroy (opts->fields);
       opts->fields = bson_copy (fields);
       return true;
    }
+
    return false;
 }
 
@@ -217,9 +220,9 @@ void
 mongoc_find_and_modify_opts_destroy (mongoc_find_and_modify_opts_t *opts)
 {
    if (opts) {
-      _mongoc_bson_destroy_if_set (opts->sort);
-      _mongoc_bson_destroy_if_set (opts->update);
-      _mongoc_bson_destroy_if_set (opts->fields);
+      bson_destroy (opts->sort);
+      bson_destroy (opts->update);
+      bson_destroy (opts->fields);
       bson_destroy (&opts->extra);
       bson_free (opts);
    }
